import logging
from urllib.parse import quote
from pyvimond.http import create_http_client
from pyvimond.utils import create_basic_auth_token


class VimondImageServiceClient:
    def __init__(self, api_url, username, password, timeout=10):
        self.api_url = api_url
        self.username = username
        self.password = password
        self.timeout = timeout
        self.logger = logging.getLogger('VimondImageServiceClient')
        self.http = create_http_client()

    def _image_service_request(self, method, path):
        self.logger.info('Request %s %s', method, self.api_url + path)

        send_headers = {
            'Content-Type': 'application/json',
            'Authorization': 'Basic ' + create_basic_auth_token(self.username, self.password)
        }
        if method == "POST":
            response = self.http.post(self.api_url + path,
                                      # Omitting this empty request body results in HTTP 400 "[Invalid Json]"
                                      json={},
                                      headers=send_headers,
                                      timeout=self.timeout)
        else:
            raise ValueError('Only POST requests are supported')

        if response.status_code != 200:
            raise Exception('Request failed: status=' + str(response.status_code) + ' : ' + str(response.text),
                            response)

        return response.json()

    def create_image_pack(self):
        response = self._image_service_request('POST', '/adminAPI/imagePacks')
        return response['id']

    def send_image(self, imagepack_id, image_url, location):
        quoted_url = quote(image_url)
        path = f'/adminAPI/imagePack/{imagepack_id}/{location}/fetchImage?imageUrl={quoted_url}'
        response = self._image_service_request('POST', path)
        return response['id']

    def create_image_url(self, imagepack_id, location=None):
        if location is None:
            return f'{self.api_url}/api/v2/img/{imagepack_id}'

        return f'{self.api_url}/api/v2/img/{imagepack_id}?location={location}'



import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry


def create_http_client():
    """
    :return: an HTTP client that retries on connection errors (and limits redirects). Timeouts can apparently not be
    configured for all requests, so it's not done here.
    """
    retry_strategy = Retry(connect=3, read=0, redirect=5, status=0, other=0)
    adapter = HTTPAdapter(max_retries=retry_strategy)

    client = requests.Session()
    client.mount("https://", adapter)
    client.mount("http://", adapter)

    return client



import requests_mock
import unittest
from pyvimond.vimond_image_service_client import VimondImageServiceClient


class MyTestCase(unittest.TestCase):
    def setUp(self):
        self.vimond_image_service_client = VimondImageServiceClient('http://example.com', 'user', 'secret')

    @requests_mock.Mocker()
    def test_url_formatting_headers_and_return_value(self, m):
        m.post(
            'http://example.com/adminAPI/imagePack/123456-789/main/fetchImage?imageUrl=http%3A//example.com/image.jpg',
            json={
                'id': '123456-new'
            },
            request_headers={
                'Content-Type': 'application/json',
                'Authorization': 'Basic dXNlcjpzZWNyZXQ='
            })

        image_pack_id = self.vimond_image_service_client.send_image('123456-789', 'http://example.com/image.jpg',
                                                                    'main')
        self.assertEqual('123456-new', image_pack_id)

    @requests_mock.Mocker()
    def test_exception_contains_response_object(self, m):
        # Throw an exception
        m.post(
            'http://example.com/adminAPI/imagePack/123456-789/main/fetchImage?imageUrl=http%3A//example.com/image.jpg',
            exc=Exception('some error', {
                'status_code': 404
            }))

        try:
            self.vimond_image_service_client.send_image('123456-789', 'http://example.com/image.jpg', 'main')
        except Exception as e:
            self.assertEqual('some error', e.args[0])
            self.assertEqual(404, e.args[1]['status_code'])


if __name__ == '__main__':
    unittest.main()



import unittest
import pyvimond.utils as utils


class UtilsTest(unittest.TestCase):

    def test_create_api_metadata(self):
        metadata = {
            'foo': True,
            'bar': 'very bar',
            'baz': 42
        }

        self.assertEqual({
            'entries': {
                'foo': [{'value': True, 'lang': '*'}],
                'bar': [{'value': 'very bar', 'lang': '*'}],
                'baz': [{'value': 42, 'lang': '*'}],
            },
            'empty': True
        }, utils.create_api_metadata(metadata))

    def test_parse_metadata(self):
        asset = {
            # Normally there are other fields here, but here we only need "metadata"
            'metadata': {
                'entries': {
                    'foo': [{'value': True, 'lang': '*'}],
                    'bar': [{'value': 'very bar', 'lang': '*'}],
                    'baz': [{'value': 42, 'lang': '*'}],
                },
                'empty': True
            }
        }

        self.assertEqual({
            'foo': True,
            'bar': 'very bar',
            'baz': 42
        }, utils.parse_metadata(asset))

    def test_get_metadata_value(self):
        asset = {
            # Normally there are other fields here, but here we only need "metadata"
            'metadata': {
                'entries': {
                    'foo': [{'value': True, 'lang': '*'}],
                    'bar': [{'value': 'very bar', 'lang': '*'}],
                    'baz': [{'value': 42, 'lang': '*'}],
                },
                'empty': True
            }
        }

        self.assertTrue(utils.get_metadata_value('foo', asset))
        self.assertEqual('very bar', utils.get_metadata_value('bar', asset))
        self.assertEqual(42, utils.get_metadata_value('baz', asset))
        self.assertEqual(None, utils.get_metadata_value('ei ole', asset))

    def test_parse_category_metadata(self):
        category = {
            'metadata': {
                '@uri': '/api/metadata/category/123',
                'foo': {
                    '@xml:lang': '*',
                    '$': 'Category Foo',
                },
                'bar': {
                    '@xml:lang': '*',
                    '$': 'Category Bar',
                }
            }
        }

        self.assertEqual({
            'foo': 'Category Foo',
            'bar': 'Category Bar',
        }, utils.parse_category_metadata(category))

    def test_get_category_metadata_value(self):
        # get_category() uses this slightly unexpected format
        category = {
            'category': {
                'metadata': {
                    '@uri': '/api/metadata/category/123',
                    'string': {
                        '@xml:lang': '*',
                        '$': 'some string',
                    },
                    'int': {
                        '@xml:lang': '*',
                        '$': '42',
                    },
                    'bool': {
                        '@xml:lang': '*',
                        '$': 'false',
                    }
                }
            }
        }

        # Make sure no automatic type coercion is going on
        self.assertEqual('some string', utils.get_category_metadata_value('string', category['category']))
        self.assertEqual('42', utils.get_category_metadata_value('int', category['category']))
        self.assertEqual('false', utils.get_category_metadata_value('bool', category['category']))
        self.assertEqual(None, utils.get_category_metadata_value('baz', category['category']))

    def test_create_sumo_signature(self):
        method = 'GET'
        path = '/api/web/asset/1334686?expand=metadata&showHiddenMetadata=true'
        secret = 'secret'
        timestamp = 'Wed, 09 Dec 2020 09:48:44 +0200'

        self.assertEqual('Xm2Kr+xdb4uEp3F7COA+oj8EOpg=', utils.create_sumo_signature(method, path, secret, timestamp))

    def test_create_basic_auth_token(self):
        token = utils.create_basic_auth_token("username", "password")

        self.assertEqual('dXNlcm5hbWU6cGFzc3dvcmQ=', token)


if __name__ == '__main__':
    unittest.main()



import base64
import re

from Crypto.Hash import SHA1, HMAC


def create_api_metadata(metadata):
    api_metadata = {
        'entries': {},
        'empty': True,
    }
    for key in metadata:
        value = metadata[key]
        api_metadata['entries'][key] = [
            {'value': value, 'lang': '*'}
        ]
    return api_metadata


def parse_metadata(asset):
    metadata = asset.get('metadata', {}).get('entries', {})
    metadata_dict = {}
    for key in metadata.keys():
        metadata_dict[key] = metadata.get(key)[0]['value']
    return metadata_dict


def get_metadata_value(key, asset):
    metadata = parse_metadata(asset)
    return metadata.get(key, None)


def parse_category_metadata(category):
    metadata = category.get('metadata', {})
    metadata_dict = {}

    for key in metadata.keys():
        if key == '@uri':
            continue

        metadata_dict[key] = metadata[key]['$']

    return metadata_dict


def get_category_metadata_value(key, category):
    metadata = parse_category_metadata(category)
    return metadata.get(key, None)


def create_sumo_signature(method, path, secret, timestamp):
    plain_path = re.sub(r"\?.*", "", path)
    string_to_sign = method + "\n" + plain_path + "\n" + timestamp
    sig_hash = HMAC.new(secret.encode('utf-8'), digestmod=SHA1)
    sig_hash.update(string_to_sign.encode('utf-8'))
    return base64.b64encode(sig_hash.digest()).decode("utf-8")


def create_basic_auth_token(username, password):
    credentials_bytes = f'{username}:{password}'.encode("utf-8")
    return base64.b64encode(credentials_bytes).decode("ascii")



import time
import logging

from pyvimond.http import create_http_client
from pyvimond.utils import create_api_metadata, create_sumo_signature


class VimondClient:

    def __init__(self, api_url, user, secret, timeout=10):
        self.api_url = api_url
        self.user = user
        self.secret = secret
        self.timeout = timeout
        self.logger = logging.getLogger('VimondClient')
        self.http = create_http_client()

    def _get_auth_headers(self, method, path):
        timestamp = time.strftime("%a, %d %b %Y %H:%M:%S %z")
        signature = create_sumo_signature(method, path, self.secret, timestamp)
        auth_header = "SUMO " + self.user + ":" + signature
        return {"Authorization": auth_header,
                "x-sumo-date": timestamp}

    def _admin_request(self, method, path, body=None):
        send_headers = self._get_auth_headers(method, path)
        send_headers['Accept'] = 'application/json;v=3'
        send_headers['Content-Type'] = 'application/json;v=3'
        send_headers[
            'Accept-Language'] = '*'  # all metadata are stored with language = '*' and without this, api couldn't get any hits
        if method == "POST":
            response = self.http.post(self.api_url + path,
                                      json=body,
                                      headers=send_headers,
                                      timeout=self.timeout)
        elif method == "PUT":
            response = self.http.put(self.api_url + path,
                                     json=body,
                                     headers=send_headers,
                                     timeout=self.timeout)
        elif method == "GET":
            response = self.http.get(self.api_url + path,
                                     headers=send_headers,
                                     timeout=self.timeout)
        if response.status_code == 404:
            return None
        if response.status_code == 200:
            return response.json()
        raise Exception('Request failed: status=' + str(response.status_code) + ": " + str(response.text))

    def get_category(self, category_id):
        return self._admin_request("GET",
                                   f"/api/web/category/{str(category_id)}?expand=metadata&showHiddenMetadata=true")

    def create_asset(self, asset):
        return self._admin_request("POST", '/api/web/asset', asset)

    def get_asset(self, asset_id):
        return self._admin_request("GET", f"/api/web/asset/{str(asset_id)}?expand=metadata&showHiddenMetadata=true")

    def copy_asset(self, asset_id):
        return self._admin_request("POST", f"/api/web/asset/{asset_id}/copy", {})

    def update_asset_data(self, asset_id, payload):
        return self._admin_request("PUT", f"/api/web/asset/{str(asset_id)}", payload)

    def get_asset_metadata(self, asset_id):
        return self.get_metadata(asset_id, 'asset')

    def get_category_metadata(self, category_id):
        return self.get_metadata(category_id, 'category')

    def get_metadata(self, vman_id, vman_type):
        return self._admin_request('GET', f'/api/metadata/{vman_type}/{str(vman_id)}.json')

    def update_asset_metadata(self, asset_id, metadata):
        api_metadata = create_api_metadata(metadata)
        return self._admin_request("POST", f"/api/metadata/asset/{str(asset_id)}", api_metadata)

    def update_category_metadata(self, category_id, metadata):
        api_metadata = create_api_metadata(metadata)
        return self._admin_request("POST", f"/api/metadata/category/{str(category_id)}", api_metadata)

    def get_asset_relations(self, asset_id):
        return self._admin_request('GET', f'/api/web/asset/{asset_id}/relations')

    def get_video_files(self, platform, asset_id):
        response = self._admin_request("GET", f"/api/{platform}/asset/{asset_id}/videoFiles")
        if not response:
            return []
        return response.get('videoFiles', [])

    def get_active_video_files(self, asset_id):
        # all file types are not visible to all platforms
        web_files = self.get_video_files('web', asset_id)
        mobileapp_files = self.get_video_files('mobileapp', asset_id)
        files = web_files + mobileapp_files
        deduplicate = {}
        active_files = [file for file in files if not file['deleted']]
        for file in active_files:
            deduplicate[file['id']] = file
        deduped_files = list(deduplicate.values())
        return sorted(deduped_files, key=lambda x: x['id'])

    def find_assets_by_image_url(self, category_id, image_url_pattern):
        return self.find_assets_by_query(category_id, f'imageUrl:{image_url_pattern}')

    def find_assets_by_metadata(self, metadata_key, metadata_value):
        return self._admin_request('GET', f'/api/metadata/lookup/assets/{metadata_key}.json?value={metadata_value}')

    def find_assets_by_query(self, category_id, query):
        """
        Finds assets from the specified category using an arbitrary SOLR query string
        :param category_id: the category to search in. "root" can also be used instead of a numeric ID.
        :param query: the raw query, e.g. "assetMeta_foo_text:bar AND updateTime:[NOW-7DAYS TO NOW]"
        :return:
        """
        return self._admin_request('GET', f'/api/web/search/categories/{category_id}/assets.json?query={query}')



import json
import requests_mock
import unittest
from pyvimond.vimond_client import VimondClient


class MyTestCase(unittest.TestCase):
    def setUp(self):
        self.vimond_client = VimondClient('http://example.com', 'user', 'secret')

    @requests_mock.Mocker()
    def test_get_video_files(self, m):
        m.get('http://example.com/api/web/asset/123/videoFiles', text=json.dumps({
            'videoFiles': [
                {
                    'id': 12345
                },
                {
                    'id': 23456
                }
            ]
        }))

        video_files = self.vimond_client.get_video_files('web', 123)
        self.assertEqual(2, len(video_files))
        self.assertEqual(12345, video_files[0]['id'])
        self.assertEqual(23456, video_files[1]['id'])

    @requests_mock.Mocker()
    def test_get_active_video_files(self, m):
        # Make the "web" platform return one two files and the "mobileapp" platform return three,
        # one of which is a duplicate of one in "web" and one which is deleted. Make them come in
        # reverse order based on their ID.
        m.get('http://example.com/api/web/asset/123/videoFiles', text=json.dumps({
            'videoFiles': [
                {
                    'id': 5564,
                    'deleted': False
                },
                {
                    'id': 4445,
                    'deleted': False
                }
            ]
        }))
        m.get('http://example.com/api/mobileapp/asset/123/videoFiles', text=json.dumps({
            'videoFiles': [
                {
                    'id': 5564,
                    'deleted': False
                },
                {
                    'id': 2344,
                    'deleted': True
                },
                {
                    'id': 1235,
                    'deleted': False
                }
            ]
        }))

        # Result should be a deduplicated, sorted, combined result of the video files
        # in the "web" and "mobileapp" platforms
        active_video_files = self.vimond_client.get_active_video_files(123)
        self.assertEqual([
            {
                'id': 1235,
                'deleted': False,
            },
            {
                'id': 4445,
                'deleted': False,
            },
            {
                'id': 5564,
                'deleted': False
            }
        ], active_video_files)

    @requests_mock.Mocker()
    def test_find_assets_by_image_url(self, m):
        expected_url = 'http://example.com/api/web/search/categories/33/assets.json?query=imageUrl:*/foo/*'
        expected_return = {'foo': True}
        m.get(expected_url, text=json.dumps(expected_return))

        self.assertEqual(expected_return, self.vimond_client.find_assets_by_image_url(33, '*/foo/*'))

    @requests_mock.Mocker()
    def test_get_asset_metadata(self, m):
        # Test that the correct URL is requested
        expected_url = 'http://example.com/api/metadata/asset/123.json'
        m.get(expected_url, text=json.dumps({}))

        self.assertEqual({}, self.vimond_client.get_asset_metadata(123))

    @requests_mock.Mocker()
    def test_get_category_metadata(self, m):
        # Test that the correct URL is requested
        expected_url = 'http://example.com/api/metadata/category/123.json'
        m.get(expected_url, text=json.dumps({}))

        self.assertEqual({}, self.vimond_client.get_category_metadata(123))


if __name__ == '__main__':
    unittest.main()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyvimond",
    version="0.4.3",
    author="Sam Stenvall",
    author_email="sam.stenvall@nitor.com",
    description="Tiny client library for various Vimond APIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.mtv.fi/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "pycryptodome",
        "requests",
    ]
)
